<?php

namespace Monarch;

use Dompdf\Dompdf;
use Drupal\Core\Asset\AttachedAssets;
use Symfony\Component\HttpFoundation\Response;

/**
 * Builds a PDF Response for a controller.
 */
class PdfResponse extends Response {

  /**
   * Converts a render array into a pdf, attaching css from libraries as needed.
   *
   * @param array $build
   *   The render array to convert.
   * @param int $status
   *   The http response code.
   * @param array $headers
   *   The http response headers.
   * @param string|float[] $size
   *   \Dompdf\Dompdf paper size.
   * @param string $orientation
   *   \Dompdf\Dompdf orientation.
   */
  public function __construct(array $build, int $status = 200, array $headers = [], string|array $size = 'letter', string $orientation = 'portrait', bool $allowRemote = TRUE) {
    $render = '';
    $attachments = [];

    /**
     * @var \Drupal\Core\Render\RendererInterface $renderer
     */
    $renderer = \Drupal::service('renderer');
    $render = (string) $renderer->renderRoot($build);
    $attachments = $build['#attached'] ?? [];

    $header_keys = array_map('strtolower', array_keys($headers));

    if (!in_array('content-type', $header_keys)) {
      $headers['Content-Type'] = 'application/pdf';
    }

    if (!in_array('content-transfer-encoding', $header_keys)) {
      $headers['Content-Transfer-Encoding'] = 'Binary';
    }

    $css_render = '';

    if (!empty($attachments)) {
      $assets = new AttachedAssets();
      $assets->setLibraries($attachments['library'] ?? []);
      /** @var \Drupal\Core\Asset\AssetResolverInterface $asset_resolver */
      $asset_resolver = \Drupal::service('asset.resolver');
      $css_assets = $asset_resolver->getCssAssets($assets, FALSE);

      foreach ($css_assets as &$css_asset) {
        $css_asset['preprocessed'] = TRUE;
        $css_asset['preprocess'] = FALSE;
      }

      /** @var \Drupal\Core\Asset\CssCollectionRenderer $css_collection_renderer */
      $css_collection_renderer = \Drupal::service('asset.css.collection_renderer');
      $css_render_array = $css_collection_renderer->render($css_assets);

      foreach ($css_render_array as $key => &$element) {
        if (is_string($key) && $key[0] === '#') {
          continue;
        }

        if (isset($element['#attributes']['href']) && is_string($element['#attributes']['href']) && $element['#attributes']['href'][0] === '/') {
          $element['#attributes']['href'] = DRUPAL_ROOT . $element['#attributes']['href'];
        }
      }

      $css_render = (string) $renderer->renderPlain($css_render_array);
    }

    $html = "<!doctype html>\n<html><head>$css_render</head><body>$render</body></html>";
    $dompdf = new Dompdf();
    $dompdf->setBasePath(DRUPAL_ROOT);
    $dompdf->getOptions()->setChroot(DRUPAL_ROOT);
    $dompdf->getOptions()->set('isRemoteEnabled', $allowRemote);
    $dompdf->setBaseHost(\Drupal::request()->getHost());
    $dompdf->loadHtml($html);
    $dompdf->setPaper($size, $orientation);
    $dompdf->render();
    $render = $dompdf->output() ?? '';

    parent::__construct($render, $status, $headers);
  }

}
